"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const react_dom_1 = require("react-dom");
const use_ssr_1 = __importDefault(require("use-ssr"));
const errorMessage1 = 'You must either bind to an element or pass an event to openPortal(e).';
function usePortal(_a = {}) {
    var { closeOnOutsideClick = true, closeOnEsc = true, renderOnClickedElement, renderBelowClickedElement, bindTo, // attach the portal to this node in the DOM
    isOpen: defaultIsOpen = false, onOpen, onClose } = _a, eventHandlers = __rest(_a, ["closeOnOutsideClick", "closeOnEsc", "renderOnClickedElement", "renderBelowClickedElement", "bindTo", "isOpen", "onOpen", "onClose"]);
    const { isServer, isBrowser } = use_ssr_1.default();
    const [isOpen, makeOpen] = react_1.useState(defaultIsOpen);
    // we use this ref because `isOpen` is stale for handleOutsideMouseClick
    const open = react_1.useRef(isOpen);
    const setOpen = react_1.useCallback((v) => {
        // workaround to not have stale `isOpen` in the handleOutsideMouseClick
        open.current = v;
        makeOpen(v);
    }, []);
    const targetEl = react_1.useRef(); // this is the element you are clicking/hovering/whatever, to trigger opening the portal
    const portal = react_1.useRef(isBrowser ? document.createElement('div') : null);
    react_1.useEffect(() => {
        if (isBrowser && !portal.current)
            portal.current = document.createElement('div');
    }, [isBrowser, portal]);
    const elToMountTo = react_1.useMemo(() => {
        if (isServer)
            return;
        return (bindTo && react_dom_1.findDOMNode(bindTo)) || document.body;
    }, [isServer, bindTo]);
    const handleEvent = react_1.useCallback((func, event) => {
        if (event && event.currentTarget)
            targetEl.current = event.currentTarget;
        // i.e. onClick, etc. inside usePortal({ onClick({ portal, targetEl }) {} })
        const maybePortal = func({ portal, targetEl: targetEl.current, event });
        if (maybePortal)
            portal.current = maybePortal.current;
    }, [portal, targetEl]);
    const openPortal = react_1.useCallback((event) => {
        if (isServer)
            return;
        // for some reason, when we don't have the event argument there
        // is a weird race condition, would like to see if we can remove
        // setTimeout, but for now this works
        if (event == null && targetEl.current == null) {
            setTimeout(() => setOpen(true), 0);
            throw Error(errorMessage1);
        }
        if (event && event.nativeEvent)
            event.nativeEvent.stopImmediatePropagation();
        if (event)
            targetEl.current = event.currentTarget;
        if (!targetEl.current)
            throw Error(errorMessage1);
        const { left, top } = targetEl.current.getBoundingClientRect();
        if (onOpen)
            handleEvent(onOpen, event);
        if (renderOnClickedElement && portal.current instanceof HTMLElement) {
            portal.current.style.cssText = `
        height: 0px;
        position: absolute;
        left: ${left}px;
        top: ${top}px;
      `;
        }
        else if (renderBelowClickedElement && portal.current instanceof HTMLElement) {
            portal.current.style.cssText = `
        position: absolute;
        left: ${left};
        top: ${top};
      `;
        }
        setOpen(true);
    }, [isServer, portal, setOpen, renderBelowClickedElement, renderOnClickedElement, handleEvent, targetEl, onOpen]);
    const closePortal = react_1.useCallback((event) => {
        if (isServer)
            return;
        if (onClose)
            handleEvent(onClose, event);
        if (open.current)
            setOpen(false);
    }, [isServer, handleEvent, onClose, setOpen]);
    const togglePortal = react_1.useCallback((e) => isOpen ? closePortal(e) : openPortal(e), [isOpen, closePortal, openPortal]);
    const handleKeydown = react_1.useCallback(e => {
        var ESC = 27;
        if (e.keyCode === ESC && closeOnEsc)
            closePortal(e);
    }, [closeOnEsc, closePortal]);
    const handleOutsideMouseClick = react_1.useCallback(e => {
        if (isServer)
            return;
        if (!(portal.current instanceof HTMLElement))
            return;
        if (portal.current.contains(e.target) || e.button !== 0 || !open.current)
            return;
        if (closeOnOutsideClick)
            closePortal(e);
    }, [isServer, closePortal, closeOnOutsideClick, portal]);
    react_1.useEffect(() => {
        if (isServer)
            return;
        if (!(elToMountTo instanceof HTMLElement) || !(portal.current instanceof HTMLElement))
            return;
        const node = portal.current;
        elToMountTo.appendChild(portal.current);
        document && document.addEventListener('keydown', handleKeydown);
        document && document.addEventListener('click', handleOutsideMouseClick);
        return () => {
            document && document.removeEventListener('keydown', handleKeydown);
            document && document.removeEventListener('click', handleOutsideMouseClick);
            elToMountTo.removeChild(node);
        };
    }, [isServer, handleOutsideMouseClick, handleKeydown, elToMountTo, portal]);
    const Portal = react_1.useCallback(({ children }) => {
        if (portal.current != null)
            return react_dom_1.createPortal(children, portal.current);
        return null;
    }, [portal]);
    // this should handle all eventHandlers like onClick, onMouseOver, etc. passed into the config
    const customEventHandlers = Object
        .entries(eventHandlers)
        .reduce((acc, [handlerName, eventHandler]) => {
        acc[handlerName] = (event) => handleEvent(eventHandler, event);
        return acc;
    }, {});
    return Object.assign([openPortal, closePortal, open.current, Portal, togglePortal], Object.assign(Object.assign({ isOpen: open.current, openPortal, onMouseDown: handleKeydown, ref: targetEl, closePortal,
        togglePortal,
        Portal }, customEventHandlers), { bind: Object.assign({ onMouseDown: handleKeydown, ref: targetEl }, customEventHandlers) }));
}
exports.default = usePortal;
//# sourceMappingURL=usePortal.js.map